@@ -1,5 +1,6 @@
 ##################################### Bai ##################################
-bai.dim.opt <- function(Obj, d.max = NULL, sig2.hat = NULL)
+bai.dim.opt <- function(Obj, d.max = NULL, sig2.hat = NULL, criteria = c("PC1","PC2","PC3", "BIC3","IC1","IC2","IC3"
+			, "IPC1","IPC2","IPC3", "Eup.PC1", "Eup.IPC1"))
 	{
 	nr    <- Obj$nr
 	nc    <- Obj$nc
@@ -17,48 +18,113 @@ bai.dim.opt <- function(Obj, d.max = NULL, sig2.hat = NULL)
 	ref   <- ifelse(is.null(sig2.hat),w[(d.max+1)], sig2.hat)
 	rmw   <- w - ref
 	dmax  <- length(rmw[rmw>= 0]) - 1
+      d.max.p1.gzero <- length(w[1:(d.max+1)][w[1:(d.max+1)]>0])
 
+	Result = NULL
+	
+	if("PC1"%in%criteria) {
 	PC1 = w + ref * d.seq * ((nr+nc)/(nr*nc)) * log((nr*nc)/(nr + nc))
 	d.opt.PC1 <- order(PC1)[1]-1	
+	if(d.opt.PC1 > d.max) d.opt.PC1 = d.max
+	Result = rbind(Result, c( d.opt.PC1, w[d.opt.PC1+1], ref, min(dmax, d.max)))
+	}
+ 
+	if("PC2"%in%criteria)  {
 	PC2 = w + ref *d.seq * ((nr+nc)/(nr*nc)) * log(C)
 	d.opt.PC2 <- order(PC2)[1]-1	
+	if(d.opt.PC2 > d.max) d.opt.PC2 = d.max
+	Result = rbind(Result, c( d.opt.PC2, w[d.opt.PC2+1], ref, min(dmax, d.max)))
+	}
+	if("PC3"%in%criteria) {
 	PC3 = w + ref * d.seq * (log(C)/C)
 	d.opt.PC3 <- order(PC3)[1]-1	
-        ## number of eigenvalues greater than zero: 
-        d.max.p1.gzero <- length(w[1:(d.max+1)][w[1:(d.max+1)]>0])
-        ##
+	if(d.opt.PC3 > d.max) d.opt.PC3 = d.max	
+	Result = rbind(Result, c( d.opt.PC3, w[d.opt.PC3+1], ref, rep(min(dmax, d.max))))
+	}
+	if("BIC3"%in%criteria) {
+	BIC3 = w + ref *d.seq * ((nr+nc - ref)/(nr*nc)) *(log(nr*nc))
+	d.opt.BIC3 <- order(BIC3)[1]-1	
+	if(d.opt.BIC3 > d.max) d.opt.BIC3 = d.max
+	Result = rbind(Result, c( d.opt.BIC3, w[d.opt.BIC3+1], ref, rep(min(dmax, d.max))))
+	}
+	if("IC1"%in%criteria)  {
 	IC1 = log(w[1:d.max.p1.gzero]) + d.seq[1:d.max.p1.gzero] * ((nr+nc)/(nr*nc)) * log((nr*nc)/(nr + nc))
 	d.opt.IC1 <- order(IC1)[1]-1	
+	if(d.opt.IC1 > d.max) d.opt.IC1 = d.max
+	Result = rbind(Result, c( d.opt.IC1, w[d.opt.IC1+1], ref, rep(min(dmax, d.max))))
+	}
+	if("IC2"%in%criteria)  {
 	IC2 = log(w[1:d.max.p1.gzero]) + d.seq[1:d.max.p1.gzero] * ((nr+nc)/(nr*nc)) * log(C)
 	d.opt.IC2 <- order(IC2)[1]-1	
+	if(d.opt.IC2 > d.max) d.opt.IC2 = d.max
+	Result = rbind(Result, c( d.opt.IC2, w[d.opt.IC2+1], ref, rep(min(dmax, d.max))))
+	}
+	if("IC3"%in%criteria)  {
 	IC3 = log(w[1:d.max.p1.gzero]) + d.seq[1:d.max.p1.gzero] * (log(C)/C)
 	d.opt.IC3 <- order(IC3)[1]-1	
-
+	if(d.opt.IC3 > d.max) d.opt.IC3 = d.max
+	Result = rbind(Result, c( d.opt.IC3, w[d.opt.IC3+1], ref, rep(min(dmax, d.max))))	
+	}
+	if("IPC1"%in%criteria) {
 	IPC1 = w + ref * d.seq * (nr/(4*log(log(nr)))) *((nr+nc)/(nr*nc)) * log((nr*nc)/(nr + nc))
 	d.opt.IPC1 <- order(IPC1)[1]-1	
+	if(d.opt.IPC1 > d.max) d.opt.IPC1 = d.max
+	Result = rbind(Result, c( d.opt.IPC1, w[d.opt.IPC1+1], ref, rep(min(dmax, d.max))))
+	}
+	if("IPC2"%in%criteria) {
 	IPC2 = w + ref *d.seq * (nr/(4*log(log(nr)))) *((nr+nc)/(nr*nc)) * log(C)
 	d.opt.IPC2 <- order(IPC2)[1]-1
+	if(d.opt.IPC2 > d.max) d.opt.IPC2 = d.max
+	Result = rbind(Result, c( d.opt.IPC2, w[d.opt.IPC2+1], ref, rep(min(dmax, d.max))))	
+	} 
+	if("IPC3"%in%criteria) {
 	IPC3 = w + ref * d.seq * (nr/(4*log(log(nr)))) *((nr+nc - ref)/(nr*nc)) *(log(nr*nc))
 	d.opt.IPC3 <- order(IPC3)[1]-1
+	if(d.opt.IPC3 > d.max) d.opt.IPC3 = d.max
+	Result = rbind(Result, c( d.opt.IPC3, w[d.opt.IPC3+1], ref, rep(min(dmax, d.max))))	
+	}
+	if("Eup.PC1"%in%criteria) {
+	Eup.PC1 = w + ref * d.seq * ((nr+nc)/(nr*nc)) * log((nr*nc)/(nr + nc))
+	d.opt.Eup.PC1 <- order(Eup.PC1)[1]-1	
+	dyn.d.amx <- d.max
+	dyn.ref1 = ref
+	while(0 < d.opt.Eup.PC1 && d.opt.Eup.PC1 < dyn.d.amx){
+		dyn.d.amx = d.opt.Eup.PC1
+		dyn.ref1 = w[(dyn.d.amx+1)]
+		Eup.PC1 = w + dyn.ref1 * d.seq * ((nr+nc)/(nr*nc)) * log((nr*nc)/(nr + nc))
+		d.opt.Eup.PC1 <- order(Eup.PC1)[1]-1	
+		if(d.opt.Eup.PC1 > dyn.d.amx) d.opt.Eup.PC1 = dyn.d.amx
+		}
 
-	result <- matrix(c(d.opt.PC1, d.opt.PC2, d.opt.PC3, d.opt.IC1
-				, d.opt.IC2, d.opt.IC3, d.opt.IPC1, d.opt.IPC2
-				, d.opt.IPC3, w[d.opt.PC1+1], w[d.opt.PC2+1]
-				, w[d.opt.PC3+1], w[d.opt.IC1 + 1], w[d.opt.IC2+1]
-				, w[d.opt.IC3+1], w[d.opt.IPC1+1], w[d.opt.IPC2+1]
-				, w[d.opt.IPC3+1]), 9,2)
-	all.crit <- c("PC1", "PC2",  "PC3", "IC1"
-				, "IC2", "IC3", "IPC1", "IPC2", "IPC3")
-	Result <- data.frame(I(all.crit), result
-				, rep.int(ref, 9), rep(min(dmax, d.max), 9))
-	colnames(Result) <- c("Criterion ", "Optimal Dimension", "sd2"
-				, "sd2.hat.ref", "d.ref.max")
-	return(Result)
+	Result = rbind(Result, c(d.opt.Eup.PC1, w[d.opt.Eup.PC1+1], dyn.ref1, min(dmax, d.max)))
+	}
+	if("Eup.IPC1"%in%criteria) {
+	Eup.IPC1 =  w + ref * d.seq * (nr/(4*log(log(nr)))) *((nr+nc - ref)/(nr*nc)) *(log(nr*nc))
+	d.opt.Eup.IPC1 <- order(Eup.IPC1)[1]-1	
+	dyn.d.amx <- d.max
+	dyn.ref2 =  ref
+	while(0 < d.opt.Eup.IPC1 && d.opt.Eup.IPC1 < dyn.d.amx){
+		dyn.d.amx = d.opt.Eup.IPC1
+		dyn.ref2 = w[(dyn.d.amx+1)]
+		Eup.Eup.IPC1 = w + dyn.ref2 * d.seq * (nr/(4*log(log(nr)))) *((nr+nc - ref)/(nr*nc)) *(log(nr*nc))
+		d.opt.Eup.IPC1 <- order(Eup.IPC1)[1]-1	
+		}
+	if(d.opt.Eup.IPC1 > d.max) d.opt.Eup.IPC1 = d.max
+	Result = rbind(Result, c(d.opt.Eup.IPC1, w[d.opt.Eup.IPC1+1], dyn.ref2, min(dmax, d.max)))
+	}
+
+	all.crit <- c("PC1", "PC2",  "PC3", "BIC3", "IC1"
+				, "IC2", "IC3", "IPC1", "IPC2", "IPC3", "Eup.PC1", "Eup.IPC1")
+	result <- data.frame(I(all.crit[all.crit%in%criteria]), Result)
+
+
+	colnames(result) = c("Criterion ", "Optimal Dimension", "sd2" , "sd2.hat.ref", "d.ref.max")
+	return(result)
 	}
 
 
-B.OptDim <- function(Obj, criteria = c("PC1","PC2","PC3","IC1","IC2","IC3"
-			, "IPC1","IPC2","IPC3") , d.max = NULL, sig2.hat = NULL){
+B.OptDim <- function(Obj, criteria = c("PC1","PC2","PC3", "BIC3","IC1","IC2","IC3"
+			, "IPC1","IPC2","IPC3", "Eup.PC1", "Eup.IPC1") , d.max = NULL, sig2.hat = NULL){
 	# what is Obj?
 	if(class(Obj)=="svd.pca"|class(Obj)=="fsvd.pca") obj <- Obj
 	else{
@@ -86,13 +152,221 @@ B.OptDim <- function(Obj, criteria = c("PC1","PC2","PC3","IC1","IC2","IC3"
 				}
 			}
 		}
-	result <- bai.dim.opt(obj, d.max = d.max, sig2.hat = sig2.hat)
 	criteria <- match.arg(criteria, several.ok = TRUE)
-	return(result[result[,1] %in% criteria, ])
+	result <- bai.dim.opt(obj, d.max = d.max, sig2.hat = sig2.hat, criteria = criteria)
+	return(result)
 	}
 
 #####################################################################################################################
+abc.OptDim <- function(Obj , c.grid=NULL, n.seq=NULL, T.seq=NULL, 
+					d.max = NULL, criteria = c("ABC.IC1", "ABC.IC2", "DCPC")){
+	nr <- nrow(Obj)
+	nc <- ncol(Obj)
+	C = min(nr, nc)
 
+	if(is.null(d.max)) d.max = trunc(sqrt(min(nr, nc)))
+	d.max = d.max +1
+	if(is.null(n.seq)| length(n.seq) == 1){
+		if(length(n.seq) == 1) {
+			l.n.seq <- n.seq
+			if(l.n.seq > (nc-d.max -1)| l.n.seq < 2) stop("The given length of 'n.seq' is too large. The length should be between '2' and '(n - d.max +1)'. ", call. = FALSE)
+
+		}
+		else{
+		#l.n.seq <- min((nc-d.max -1), (nc - trunc(sqrt(nc)) -1), 30)
+		l.n.seq <- min((nc-d.max -1),  trunc(sqrt(nc)), 30)
+		}	
+	}
+	else{
+	l.n.seq <- length(n.seq)
+	if(any(n.seq) > nc | any(n.seq < (d.max+1))) stop("The sequence 'n.seq' contains dimension(s) larger than 'n' or smaller than '(d.max +1).", call. = FALSE)
+	}
+
+	if(is.null(T.seq)| length(T.seq) == 1){
+		if(length(T.seq) == 1) {
+			l.T.seq <- T.seq
+			if(l.T.seq > (nr-d.max -1)| l.T.seq < 2) stop("The given length of 'T.seq' is too large. The length should be between '2' and '(T - d.max +1)'. ", call. = FALSE)
+		}
+		else{
+		#l.T.seq <- min((nr-d.max -1), (nr- trunc(sqrt(nr))-1), 30)
+		l.T.seq <- min((nr-d.max -1), trunc(sqrt(nr)), 30)
+		}	
+	}
+	else{
+	l.T.seq <- length(T.seq)
+	if(any(T.seq) > nr | any(T.seq < (d.max+1))) stop("The sequence 'T.seq' contains dimension(s) larger than 'T' or smaller than '(d.max +1).", call. = FALSE)
+	}
+	if(l.T.seq == l.n.seq){
+		if(length(T.seq) < 2) T.seq <- nr:(nr-l.T.seq+1)
+		else T.seq <- trunc(sort(T.seq, decreasing = TRUE))
+		if(length(n.seq) < 2) n.seq <- nc:(nc-l.n.seq+1)
+		else n.seq <- trunc(sort(n.seq, decreasing = TRUE))
+	}
+	else{
+		if(l.n.seq > l.T.seq){
+		if(length(n.seq) < 2) n.seq <- nc:(nc-l.n.seq+1)
+		else n.seq <- trunc(sort(n.seq, decreasing = TRUE))
+
+		if(length(T.seq) < 2)
+		T.seq <- trunc(seq.int(nr, (nr - l.T.seq), length.out = l.n.seq))
+		else
+		T.seq <- trunc(seq.int(max(T.seq), min(T.seq), length.out = l.n.seq))
+		}
+		else{
+		if(length(T.seq) < 2)
+		T.seq <- nr:(nr-l.T.seq+1)
+		else T.seq <- trunc(sort(T.seq, decreasing = TRUE))
+
+		if(length(n.seq) < 2) 
+		n.seq <- trunc(seq.int(nc, (nc-l.n.seq), length.out = l.T.seq))
+		else
+		n.seq <- trunc(seq.int(max(n.seq), min(n.seq), length.out = l.T.seq))
+		}
+	}
+	FUN.Eig <- function(Obj, n.seq, T.seq, j){
+		nc.j <- n.seq[j]
+		nr.j <- T.seq[j]	
+		Obj.j <- Obj[1:nr.j, 1:nc.j]
+
+		if(nc.j < nr.j) Q.j <- crossprod(Obj.j)/(nr.j*nc.j)
+		else Q.j <- tcrossprod(Obj.j)/(nr.j*nc.j)
+		eig.j <- eigen(Q.j, only.values =  TRUE)[[1]]
+		eig.j <- c(eig.j[1:d.max], sum(eig.j[-(1:d.max)]))
+	}
+	leng.seq <- length(n.seq)
+	Eig.seq <- t(sapply(1:leng.seq, FUN.Eig, Obj = Obj, n.seq=n.seq, T.seq = T.seq))	
+	Eig.Seq <- Eig.seq[, -(d.max+1)]
+	rSumE <- rowSums(Eig.seq) 
+	rCumE <- t(apply(Eig.Seq, 1, cumsum))
+	W <- cbind(rSumE, (rSumE - rCumE))
+	log.W <- log(W)
+
+	pen1.seq <- ((n.seq+T.seq)/(n.seq*T.seq)) * log((n.seq*T.seq)/(n.seq + T.seq))
+	pen2.seq <- ((n.seq+T.seq)/(n.seq*T.seq)) * log(sqrt(min(nr, nc)))
+	if(is.null(c.grid)) c.grid <- seq.int(0, 5, length.out= 128)
+
+	FUN.dims <- function(log.W, m, c.grid, pen.seq){ #c=0.05
+	logw <- log.W[m,]
+	g <- pen.seq[m]*c.grid
+	Cirt <- logw%*%t(rep(1, length(c.grid))) + 0:(d.max)%*%t(g) 
+	dims <- apply(Cirt, 2, order)[1, ] -1
+	dims
+	}
+
+	Result = NULL
+	if("ABC.IC1"%in%criteria) {
+	Dims <- sapply(1:leng.seq, FUN.dims, log.W = log.W, c.grid = c.grid, pen.seq =pen1.seq)
+	VarD <- apply(Dims, 1, var)
+	if(all(Dims[, 1][VarD==min(VarD)] == d.max)) dhat = d.max -1
+	else {
+		dhat <- Dims[, 1][VarD==min(VarD)&Dims[, 1]<(d.max-1)]
+		ldhat <- length(dhat)
+			if(ldhat > 1){
+				l=1
+				repeat{
+				if(dhat[l]==dhat[l+1]| l== ldhat| all(dhat ==dhat[1])) break
+				l= l+1
+				}
+			dhat = dhat[l]
+			}
+
+		}
+	ref <- mean(c.grid[Dims[, 1] == dhat])
+	Result = rbind(Result, c(dhat, W[1, dhat+1], ref, d.max, min(c.grid), max(c.grid), length(c.grid)))
+	}
+	if("ABC.IC2"%in%criteria) {
+	Dims2 <- sapply(1:leng.seq, FUN.dims, log.W = log.W, c.grid = c.grid, pen.seq =pen2.seq)
+	VarD2 <- apply(Dims2, 1, var)
+	if(all(Dims2[, 1][VarD2==min(VarD2)] == d.max)) dhat2 = d.max -1
+
+	else {
+		dhat2 <- Dims2[, 1][VarD2==min(VarD2)&Dims2[, 1]< (d.max-1)]
+		ldhat2 <- length(dhat2)
+			if(ldhat2 > 1){
+				l=1
+				repeat{
+				if(dhat2[l]==dhat2[l+1]| l== ldhat2| all(dhat2==dhat2[1])) break
+				l= l+1
+				}
+			dhat2 = dhat2[l]
+			}
+
+		}
+	ref2 <- mean(c.grid[Dims2[, 1] == dhat2])
+	Result = rbind(Result, c(dhat2, W[1,dhat2+1], ref2, d.max, min(c.grid), max(c.grid), length(c.grid)))
+	}
+
+	if("DCPC"%in%criteria) {
+
+
+#	FUN.EigCV <- function(Obj, i){
+#		if(i == 1) Obj.j <- Obj
+#		else {
+#			Obj.j <- Obj[-i, -i]
+#			nr = nr-1
+#			nc = nc-1
+#			}
+#
+#		if(nc < nr) Q.j <- crossprod(Obj.j)/(nr*nc)
+##		else Q.j <- tcrossprod(Obj.j)/(nr*nc)
+#		eig.j <- eigen(Q.j, only.values =  TRUE)[[1]]
+#		eig.j <- c(eig.j[1:d.max], sum(eig.j[-(1:d.max)]))
+#	}
+#	leng.seq <- C
+#	Eig.seq <- t(sapply(1:leng.seq, FUN.EigCV, Obj = Obj))	
+#	Eig.Seq <- Eig.seq[, -(d.max+1)]
+#	rSumE <- rowSums(Eig.seq) 
+#	rCumE <- t(apply(Eig.Seq, 1, cumsum))
+#	W <- cbind(rSumE, (rSumE - rCumE))
+#	C.seq <- c(C, rep((C-1), (leng.seq-1)))
+#
+	C.seq <- apply(cbind(n.seq, T.seq), 1, min)^{0.5}
+	FUN.seq <- function(W, m, alpha.grid){#m=1
+		w <- W[m, ]
+		g <- C.seq [m]^{-2*(alpha.grid)}
+		Crit <- w%*%(1 - t(g) ) + 0:(length(w)-1)%*%t(g)
+		#Crit <- w%*%t(alpha.grid) + (0:(length(w)-1)%*%t(g))*rep(1, length(w))%*%(1-t(alpha.grid))
+		dims.agrid <- apply(Crit, 2, function(x) order(x)[1] -1)
+		dims <- dims.agrid
+		dims
+	}
+
+  ## find smalles and largest alpha
+	start.alpha.grid <- seq.int(0.001, 0.999, length.out= 512)
+	dims1 <- FUN.seq(W, 1, start.alpha.grid)
+	low.alpha <- ifelse(sum(start.alpha.grid[dims1==0])> 0, min(start.alpha.grid[dims1==0]), min(start.alpha.grid))
+	upp.apla <- ifelse(sum(start.alpha.grid[dims1<=(d.max-1)])> 0, max(start.alpha.grid[dims1<=(d.max-1)]), mean(start.alpha.grid))
+	alpha.grid <- seq.int(low.alpha, upp.apla, length.out= 128)#a.grid
+
+
+	Dims3 <- sapply(1:leng.seq, FUN.seq, W= W, alpha.grid = alpha.grid)
+	VarD3 <- apply(Dims3, 1, var)
+	adapmeand <- median(Dims3[VarD3==min(VarD3) , ])
+	adapVar <- apply(Dims3, 1, function(x) sum(abs(x -adapmeand)))
+	ref3 <- mean(min(adapVar))
+	dhat3 <-  Dims3[adapVar==ref3, 1][1]
+
+par(mfcol= c(1,4))
+	plot(alpha.grid, VarD3)
+	plot(alpha.grid, adapVar)
+	plot(alpha.grid, Dims3[, 1])
+abline(h=adapmeand)
+	points(alpha.grid[adapVar==ref3], Dims3[adapVar==ref3, 1], col= "red")
+	matplot(W)
+
+
+	Result = rbind(Result, c(dhat3, W[1, dhat3+1], ref3, d.max, min(alpha.grid), max(alpha.grid), length(alpha.grid)))	
+	}
+
+
+	all.crit = c("ABC.IC1", "ABC.IC2", "DCPC")
+	result <- data.frame(I(all.crit[all.crit%in%criteria]), Result)
+	colnames(result) = c("Criterion ", "Optimal Dimension", "sd2" , "ref.grid", "d.max", "grid.min", "grid.max", "grid.length")
+	return(result)
+} 
+
+
+#####################################################################################################################
 onatski.dim.opt <- function(svd.pca.obj, d.max = NULL)
 	{
 	nr      <- svd.pca.obj$nr
@@ -102,43 +376,22 @@ onatski.dim.opt <- function(svd.pca.obj, d.max = NULL)
 	max.rk  <- length(ev)
 
 	exa.ev  <- c(sum(ev), ev)
-	d.max   <- ifelse(is.null(d.max), max.rk-5, min(d.max, (max.rk-5)))	
-	j       <- max(1, (d.max - 1))
-
+	d.max   <- ifelse(is.null(d.max), round(sqrt(max.rk)), min(d.max, (max.rk-5)))
+	if(d.max < 1) stop("the data dimension is not sufficient to run the method of Onatski (2009)")
+	j       <- d.max - 1
+	repeat{
 	c.reg   <- as.matrix(seq((j - 1), (j + 3))^{(2/3)}, 4, 1)
 	delta   <- 2* abs(coef(lm.fit(c.reg, ev[j:(j+4)])))
-	dist.ev <- exa.ev[1:(max.rk-1)] -  exa.ev[2:max.rk] - delta
+	dist.ev <- -diff(ev[1:(j+2)]) - delta
+	dhat    <- sum(dist.ev > 0)
+	if(j == dhat| dhat == 0) break 
+	else j = dhat
+	}
 
-	FUN.o.dim <- function(o.dist.ev, d)
-		{
-		if(o.dist.ev[d] < 0) (d - 2)
-		else {
-			if (d == (max.rk-1)){
-				(d - 2)
-				warning(expression("ED faild to find an appropriate dimension")) 
-				}
-			else FUN.o.dim(o.dist.ev, (d+1))
-			}
-		}
-	
-	d.opt.i <- FUN.o.dim(dist.ev, 1)
 
-	if(d.opt.i==d.max) {
-			result <- data.frame(I("ED")
-					, matrix(c(d.opt.i, w[d.opt.i+1]), 1, 2))
-			colnames(result) <- c("Criterion", "Optimal Dimension", "sd2")
-			return(result)
-			}
-	else {
-		if (d.opt.i == (max.rk-1)) {
-			result <- data.frame(I("ED")
-					, matrix(c(d.opt.i, w[d.opt.i+1]), 1, 2))
-			colnames(result) <- c("Criterion", "Optimal Dimension", "sd2")
-			return(result)
-			warning(expression("ED faild to find an appropriate dimension")) 
-			}
-		else onatski.dim.opt(svd.pca.obj, d.opt.i)
-		}
+	result <- data.frame(I("ED"), matrix(c(dhat, w[dhat+1]), 1, 2))
+	colnames(result) <- c("Criterion", "Optimal Dimension", "sd2")
+	result
 	}
 
 O.OptDim <- function(Obj, d.max = NULL){
@@ -434,16 +687,20 @@ KSS.OptDim <- function(Obj,
 
 
 EstDim <- function(Obj, 
-                   dim.criterion = c("PC1", "PC2", "PC3",
+                   dim.criterion = c("PC1", "PC2", "PC3", "BIC3",
                      "IC1", "IC2", "IC3",
                      "IPC1","IPC2", "IPC3",
+			   "ABC.IC1", "ABC.IC2", 
                      "KSS.C",
                      "ED",  "ER",  "GR"),
                    d.max,
                    factor.dim,
                    sig2.hat,
-			 spar = NULL,
-                   level = 0.01
+			 spar,
+                   level = 0.01,
+			 c.grid = seq(0, 5, length.out = 128),
+			 T.seq,
+			 n.seq
                    )
   {
   ## missing parameters
@@ -451,6 +708,8 @@ EstDim <- function(Obj,
   	 if(missing(d.max))      d.max       <- NULL
  	 if(missing(sig2.hat))   sig2.hat    <- NULL
 	 if(missing(spar))       spar        <- NULL
+	 if(missing(T.seq))	 T.seq 	 <- NULL
+	 if(missing(n.seq))	 n.seq 	 <- NULL
 
   ## estimation 
 	 criteria <- match.arg(dim.criterion)
@@ -462,6 +721,8 @@ EstDim <- function(Obj,
                             , d.max = d.max, sig2.hat = sig2.hat)),
                           PC3  = try(B.OptDim(Obj, criteria = c("PC3")
                             , d.max = d.max, sig2.hat = sig2.hat)),
+                          BIC3  = try(B.OptDim(Obj, criteria = c("BIC3")
+                            , d.max = d.max, sig2.hat = sig2.hat)),
                           IC1  = try(B.OptDim(Obj, criteria = c("IC1")
                             , d.max = d.max, sig2.hat = sig2.hat)),
                           IC2  = try(B.OptDim(Obj, criteria = c("IC2")
@@ -474,6 +735,16 @@ EstDim <- function(Obj,
                             , d.max = d.max, sig2.hat = sig2.hat)),
                           IPC3 = try(B.OptDim(Obj, criteria = c("IPC3")
                             , d.max = d.max, sig2.hat = sig2.hat)),
+                          Eup.PC1 = try(B.OptDim(Obj, criteria = c("Eup.PC1")
+                            , d.max = d.max, sig2.hat = sig2.hat)),
+                          Eup.IPC1 = try(B.OptDim(Obj, criteria = c("Eup.IPC1")
+                            , d.max = d.max, sig2.hat = sig2.hat)),
+                          ABC.IC1 = try(abc.OptDim(Obj, criteria = c("ABC.IC1")
+                            , d.max = d.max, c.grid = c.grid, n.seq = n.seq, T.seq = T.seq)),
+                          ABC.IC2 = try(abc.OptDim(Obj, criteria = c("ABC.IC2")
+                            , d.max = d.max, c.grid = c.grid, n.seq = n.seq, T.seq = T.seq)),
+                          DCPC = try(abc.OptDim(Obj, criteria = c("DCPC")
+                            , d.max = d.max, c.grid = c.grid, n.seq = n.seq, T.seq = T.seq)),
                           ED   = try(O.OptDim(Obj, d.max = d.max)),				
                           ER   = try(RH.OptDim(Obj, criteria = c("ER")
                             , d.max = d.max)),
@@ -488,21 +759,21 @@ EstDim <- function(Obj,
 
 ############## Est dim compair  ######################################################################################
 
-
-
 OptDim.default <- function(Obj, 
-                   criteria = c("PC1", "PC2", "PC3",
+                 criteria = c("PC1", "PC2", "PC3", "BIC3",
                      "IC1", "IC2", "IC3",
                      "IPC1","IPC2", "IPC3",
+			   "ABC.IC1", "ABC.IC2",
                      "KSS.C",
                      "ED",  "ER",  "GR"),
 			standardize = FALSE,
 			d.max,
 			sig2.hat,
 			spar,
-			level = 0.01
-                   )
-  {
+			level = 0.01,
+			c.grid = seq(0, 5, length.out = 128),
+			T.seq,
+			n.seq){
   ## if Eup or KSS classes
 	if(class(Obj)=="KSS"| class(Obj)== "Eup") Obj <- Obj$unob.fact.stru + Obj$residuals
 	else is.regular.panel(Obj, stopper = TRUE)
@@ -511,45 +782,66 @@ OptDim.default <- function(Obj,
   	 if(missing(d.max))      d.max       <- NULL
  	 if(missing(sig2.hat))   sig2.hat    <- NULL
 	 if(missing(spar))       spar        <- NULL
+	 if(missing(c.grid))	 c.grid 	 <- NULL
+	 if(missing(T.seq))	 T.seq 	 <- NULL
+	 if(missing(n.seq))	 n.seq 	 <- NULL
+
 
   ## standardize
-	 if(standardize) Obj <- standardize(Obj)
+	if(standardize) Obj <- standardize(Obj)
 
   ## estimation 
-	 criteria <- match.arg(criteria, several.ok = TRUE)
-         
- 	 est.dim       <- function(criteria, d.max, sig2.hat, level){
-				switch(criteria,
-                          PC1  = try(B.OptDim(Obj, criteria = c("PC1")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          PC2  = try(B.OptDim(Obj, criteria = c("PC2")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          PC3  = try(B.OptDim(Obj, criteria = c("PC3")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          IC1  = try(B.OptDim(Obj, criteria = c("IC1")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          IC2  = try(B.OptDim(Obj, criteria = c("IC2")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          IC3  = try(B.OptDim(Obj, criteria = c("IC3")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          IPC1 = try(B.OptDim(Obj, criteria = c("IPC1")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          IPC2 = try(B.OptDim(Obj, criteria = c("IPC2")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          IPC3 = try(B.OptDim(Obj, criteria = c("IPC3")
-                            , d.max = d.max, sig2.hat = sig2.hat)),
-                          ED   = try(O.OptDim(Obj, d.max = d.max)),				
-                          ER   = try(RH.OptDim(Obj, criteria = c("ER")
-                            , d.max = d.max)),
-                          GR   = try(RH.OptDim(Obj, criteria = c("GR")
-                            , d.max = d.max)),
-                          KSS.C  = try(KSS.OptDim(Obj, criteria = c("KSS.C")
-				    , sig2.hat = sig2.hat, alpha=level
-                             , factor.dim= NULL, d.max=d.max, spar = spar)[[1]])
-                          )
-	}
+	criteria <- match.arg(criteria, several.ok = TRUE)
+ 
+	checkBai <- criteria%in%c("PC1", "PC2", "PC3", "BIC3",
+                     	      "IC1", "IC2", "IC3",
+                     		"IPC1","IPC2", "IPC3", "Eup.PC1", "Eup.IPC1")
+			if(any(checkBai)){
+				BaiS <- try(B.OptDim(Obj, criteria =  criteria[checkBai], 
+                      	d.max = d.max, sig2.hat = sig2.hat))	
+			}
 
-	Result <- sapply(criteria, est.dim, d.max = d.max, sig2.hat = sig2.hat, level = level, simplify = FALSE)
+	checkOnat <- criteria%in%c("ED")
+			if(any(checkOnat)){
+				OnatS <- try(O.OptDim(Obj, d.max = d.max))
+			}
+
+	checkRH <- criteria%in%c("ER", "GR")
+			if(any(checkRH)){
+				RHS <- try(RH.OptDim(Obj, criteria = criteria[checkRH], 
+				d.max = d.max))
+			}
+
+	checkKSS <- criteria%in%c("KSS.C")
+			if(any(checkKSS)){
+				KSSS <- try(KSS.OptDim(Obj, criteria = c("KSS.C"), 
+				sig2.hat = sig2.hat, alpha=level, 
+                        factor.dim= NULL, d.max=d.max, spar = spar)[[1]])
+			}
+
+	checkABC <- criteria%in%c("ABC.IC1", "ABC.IC2", "DCPC")
+			if(any(checkABC)){
+				ABCS <- try(abc.OptDim(Obj, criteria =  criteria[checkABC], 
+                      	c.grid = c.grid, n.seq = n.seq, T.seq= T.seq, d.max = d.max))	
+			}
+
+       ll <- length(criteria)
+	 Result <- vector(mode = "list", ll)  
+	 for(l in 1:ll) {#l=1
+		if(checkBai[l]) Result[[l]] <- BaiS[ BaiS[ ,1] == criteria[l],]
+		else{
+			if(checkOnat[l]) Result[[l]] <- OnatS
+			else{
+				if(checkRH[l]) Result[[l]] <- RHS[ RHS[ ,1] == criteria[l], ]
+				else{
+					if(checkKSS[l]) Result[[l]] <- KSSS
+					else{
+					if(checkABC[l]) Result[[l]] <- ABCS[ABCS[ ,1] == criteria[l], ]
+					}
+				}
+			}
+		}
+	}
 	summary <- sapply(1:length(criteria), function(l) Result[[l]][,1:2])
 	Summary <- matrix(as.numeric(summary[2,]), ncol(summary))
 	colnames(Summary) <- " "
@@ -557,11 +849,14 @@ OptDim.default <- function(Obj,
 	Summary  <- t(Summary )
 	Result$summary <- Summary 
 	Result$criteria <- criteria
-	Result$BaiNgC <- criteria%in%c("PC1", "PC2", "PC3", "IC1", "IC2", "IC3")
+	Result$BaiNgC <- criteria%in%c("PC1", "PC2", "PC3", "BIC3", "IC1", "IC2", "IC3")
 	Result$BaiC   <- criteria%in%c("IPC1", "IPC2", "IPC3")
-	Result$KSSC    <- criteria%in%c("KSS.C")
-	Result$OnatC  <- criteria%in%c("ED")
-	Result$RHC    <- criteria%in%c("ER", "GR")
+	Result$KSSC   <- checkKSS
+	Result$OnatC  <- checkOnat
+	Result$RHC    <- checkRH
+	Result$EupC   <- criteria%in%c("Eup.PC1", "Eup.IPC1")
+	Result$ABCC   <- criteria%in%c("ABC.IC1", "ABC.IC2")
+	Result$BadaC  <- criteria%in%c("DCPC")
 	Result$cl     <- match.call()
 	Result$obj    <- Obj
 	structure(Result, class = "OptDim")
@@ -570,16 +865,20 @@ OptDim.default <- function(Obj,
 
 # ####################### Methods ########################
 	OptDim <- function(Obj, 
-                   criteria = c("PC1", "PC2", "PC3",
+                 criteria = c("PC1", "PC2", "PC3", "BIC3",
                      "IC1", "IC2", "IC3",
                      "IPC1","IPC2", "IPC3",
+			   "ABC.IC1", "ABC.IC2", 
                      "KSS.C",
                      "ED",  "ER",  "GR"),
 			standardize = FALSE,
 			d.max,
 			sig2.hat,
 			spar,
-			level = 0.01){ UseMethod("OptDim")}
+			level = 0.01,
+			c.grid = seq(0, 5, length.out = 128),
+			T.seq,
+			n.seq){ UseMethod("OptDim")}
 
 
 ## Print
@@ -592,7 +891,7 @@ print.OptDim <- function(x,...){
 
 	if(sum(x$KSSC) > 0){
 	cat("\n---------")
-	cat("\nCriterion of Kneip, Sickles, and Song (2009):\n\n")
+	cat("\nCriterion of Kneip, Sickles, and Song (2012):\n\n")
 		KSS2009 <- numeric(0)
 		for(l in 1:ll) if(x$KSSC[l]) KSS2009 <- rbind(KSS2009, x[[l]][,1:2])
 		dimKSS2009 <- matrix(KSS2009[, 2], 1, length(KSS2009[, 2]))
@@ -614,8 +913,8 @@ print.OptDim <- function(x,...){
 	}
 	if(sum(x$RHC) > 0){
 	cat("\n--------")
-	if(sum(x$RHC) > 1) cat("\nCriteria of Ahn and Horenstein (2008):\n\n")
-	else cat("\nCriterion of Ahn and Horenstein (2008):\n\n") 
+	if(sum(x$RHC) > 1) cat("\nCriteria of Ahn and Horenstein (2013):\n\n")
+	else cat("\nCriterion of Ahn and Horenstein (2013):\n\n") 
 		AH2008 <- numeric(0)
 		for(l in 1:ll) if(x$RHC[l]) AH2008 <- rbind(AH2008, x[[l]][,1:2])
 		dimAH2008 <- matrix(AH2008[, 2], 1, length(AH2008[, 2]))
@@ -644,6 +943,38 @@ print.OptDim <- function(x,...){
 		rownames(dimO2009) <- " "
 		print(dimO2009)
 	}
+	if(sum(x$ABCC) > 0){
+		ABC <- numeric(0)
+		for(l in 1:ll) if(x$ABCC[l]) ABC <- rbind(ABC, x[[l]][,1:2])
+		dimABC <- matrix(ABC[, 2], 1, length(ABC[, 2]))
+		colnames(dimABC) <- ABC[, 1]
+		rownames(dimABC) <- " "
+	cat("\n---------")
+	if(sum(x$ABCC) > 1) cat("\nABC calibration of IC1 and IC2 (Alessi et al. (2010)):\n\n") 
+	else cat(paste("\nABC calibration of", ifelse(colnames(dimABC) == "ABC.IC1", "IC1", "IC2"), "(Alessi et al. (2010)):\n\n"))
+	print(dimABC)
+	}
+	if(sum(x$EupC) > 0){
+		B2013 <- numeric(0)
+		for(l in 1:ll) if(x$EupC[l]) B2013 <- rbind(B2013, x[[l]][,1:2])
+		dimB2013 <- matrix(B2013[, 2], 1, length(B2013[, 2]))
+		colnames(dimB2013) <- B2013[, 1]
+		rownames(dimB2013) <- " "
+	cat("\n---------")
+	if(sum(x$EupC) > 1) cat("\nEup calibration of PC1 and IPC1 (Bada and Kneip (2013)):\n\n") 
+	else cat(paste("\nEup calibration of", ifelse(colnames(dimB2013) == "Eup.PC1", "PC1", "IPC1"), "(Bada and Kneip (2013)):\n\n"))
+	print(dimB2013)
+	}
+	if(sum(x$BadaC) > 0){
+		Bada <- numeric(0)
+		for(l in 1:ll) if(x$BadaC[l]) Bada <- rbind(Bada, x[[l]][,1:2])
+		dimBada <- matrix(Bada[, 2], 1, length(Bada[, 2]))
+		colnames(dimBada) <- Bada[, 1]
+		rownames(dimBada) <- " "
+	cat("\n---------")
+	cat("\nData Adjusted Penalty of Bada (2013):\n\n") 
+	print(dimBada)
+	}
 
  }
 	
@@ -657,15 +988,15 @@ summary.OptDim <- function(object,...){
 
 	if(sum(object$KSSC) > 0){
 	cat("\n---------")
-	cat("\nSequential testing of Kneip, Sickles, and Song (2009):\n\n")
+	cat("\nSequential testing of Kneip, Sickles, and Song (2012):\n\n")
 		KSS2009 <- numeric(0)
 		for(l in 1:ll) if(object$KSSC[l]) KSS2009 <- rbind(KSS2009, object[[l]])
 		dimKSS2009 <- KSS2009[, c(2, 6)]
 		colnames(dimKSS2009) <- c("Estimate", "Level")
 		rownames(dimKSS2009) <- " KSS.C"
-		print(round(dimKSS2009,3))
+		print(signif(dimKSS2009,3))
 		cat("\n\nUsed Std Err: ")
-		cat(round(KSS2009[1, 5], 3))
+		cat(signif(KSS2009[1, 5], 3))
 	}
 	
 	if(sum(object$BaiNgC) > 0){
@@ -677,21 +1008,21 @@ summary.OptDim <- function(object,...){
 		dimBN2002 <- BN2002[, 2:3]
 		colnames(dimBN2002) <- c("Estimate", "Std Err")
 		rownames(dimBN2002) <- BN2002[, 1]
-		print(round(dimBN2002, 3))
-		cat("\n\nUsed d.max:")
+		print(signif(dimBN2002, 3))
+		cat("\n\nUsed d.max: ")
 		cat(BN2002[1, 5])
 	}
 	if(sum(object$RHC) > 0){
 	cat("\n\n--------")
-	if(sum(object$RHC) > 1) cat("\nCriteria of Ahn and Horenstein (2008):\n\n")
-	else cat("\nCriterion of Ahn and Horenstein (2008):\n\n") 
+	if(sum(object$RHC) > 1) cat("\nCriteria of Ahn and Horenstein (2013):\n\n")
+	else cat("\nCriterion of Ahn and Horenstein (2013):\n\n") 
 		AH2008 <- numeric(0)
 		for(l in 1:ll) if(object$RHC[l]) AH2008 <- rbind(AH2008, object[[l]])
 		dimAH2008 <- AH2008[, 2:3]
 		colnames(dimAH2008) <-  c("Estimate", "Std Err")
 		rownames(dimAH2008) <- AH2008[,1]
-		print(round(dimAH2008, 3))
-		cat("\n\nUsed d.max:")
+		print(signif(dimAH2008, 3))
+		cat("\n\nUsed d.max: ")
 		cat(AH2008[1, 4])	
 	}
 
@@ -704,8 +1035,8 @@ summary.OptDim <- function(object,...){
 		dimB2004 <- B2004[, 2:3]
 		colnames(dimB2004) <- c("Estimate", "Std Err")
 		rownames(dimB2004) <- B2004[, 1]
-		print(round(dimB2004, 3))
-		cat("\n\nUsed d.max:")
+		print(signif(dimB2004, 3))
+		cat("\n\nUsed d.max: ")
 		cat(B2004[1, 5])
 	}
 
@@ -717,10 +1048,55 @@ summary.OptDim <- function(object,...){
 		dimO2009 <- O2009[, 2:3]
 		colnames(dimO2009) <- c("Estimate", "Std Err")
 		rownames(dimO2009) <- O2009[1, 1]
-		print(round(dimO2009, 3))
+		print(signif(dimO2009, 3))
 	}
 
 
+	if(sum(object$ABCC) > 0){
+		ABC <- numeric(0)
+		for(l in 1:ll) if(object$ABCC[l]) ABC <- rbind(ABC, object[[l]])
+		dimABC <- ABC[, 2:3]
+		colnames(dimABC) <- c("Estimate", "Std Err")
+		rownames(dimABC) <- ABC[, 1]
+
+	cat("\n\n---------")	
+	if(sum(object$ABCC) > 1) cat("\nABC calibration of IC1 and IC2 (Alessi and all (2010)):\n\n") 
+	else cat(paste("\nABC calibration of", ifelse(rownames(dimABC) == "ABC.IC1", "IC1", "IC2"), "(Alessi and all (2010)):\n\n"))
+		print(signif(dimABC, 3))
+		cat("\n\nUsed d.max : ")
+		cat(ABC[1, 5])
+		cat(paste("\nCalibration grid between", ABC[1, 6], "and", ABC[1, 7], "with a length of", ABC[1, 8], "."))
+	}
+
+	if(sum(object$EupC) > 0){
+		B2013 <- numeric(0)
+		for(l in 1:ll) if(object$EupC[l]) B2013 <- rbind(B2013, object[[l]])
+		dimB2013 <- B2013[, 2:3]
+		colnames(dimB2013) <- c("Estimate", "Std Err")
+		rownames(dimB2013) <- B2013[, 1]
+
+	cat("\n\n---------")	
+	if(sum(object$EupC) > 1) cat("\nEup calibration of PC1 and IPC1 (Bada and Kneip (2013)):\n\n") 
+	else cat(paste("\nEup calibration of", ifelse(rownames(dimB2013) == "Eup.PC1", "PC1", "IPC1"), "(Bada and Kneip (2013)):\n\n"))
+		print(signif(dimB2013, 3))
+		cat("\n\nUsed d.max: ")
+		cat(B2013[1, 5])
+	}
+
+	if(sum(object$BadaC) > 0){
+		Bada <- numeric(0)
+		for(l in 1:ll) if(object$BadaC[l]) Bada <- rbind(Bada, object[[l]])
+		dimBada <- Bada[, 2:3]
+		colnames(dimBada) <- c("Estimate", "Std Err")
+		rownames(dimBada) <- Bada[, 1]
+	cat("\n\n---------")
+	cat("\nData Adjusted Penalty of Bada (2013):\n\n") 
+	print(signif(dimBada), 3)
+	cat("\n\nUsed d.max : ")
+	cat(Bada[1, 5])
+	cat(paste("\nCalibration grid between", Bada[1, 6], "and", Bada[1, 7], "with a length of", Bada[1, 8], "."))
+	}
+
  }
 
 plot.OptDim <- function(x, main, border, col, ...){
@@ -768,6 +1144,3 @@ plot.OptDim <- function(x, main, border, col, ...){
 	}
 	text(rep(d.max, length(dims)), (yycoreds[(dims+1)]+ 0.01) , critperdim, adj = 1, cex = 1.1)
 	}
-
-
-
